#!/usr/bin/env python

from dataclasses import dataclass, field
from datetime import datetime, date
import json
import logging
import os
import pathlib
import platform
import shutil
from logging.handlers import TimedRotatingFileHandler

from dacite import from_dict
import hikari
import lightbulb

from nidibot.nitrado import Nitrado


@dataclass
class NidibotConnectionConfiguration:
    discord_bot_token: str = ""
    nitrado_api_token: str = ""


@dataclass
class NidibotConfiguration:
    connection: NidibotConnectionConfiguration = field(
        default_factory=NidibotConnectionConfiguration
    )
    discord_admin_users: list = field(default_factory=list)


class Nidibot:
    def __init__(self, working_folder_path: str = None):
        if working_folder_path is None:
            return

        self.__working_folder_path = working_folder_path
        self.__configuration: NidibotConfiguration = NidibotConfiguration()

        #
        # Enable daily logging both to file and stdout.
        #
        log_directory = os.path.join(self.__working_folder_path, "logs")
        pathlib.Path(log_directory).mkdir(parents=True, exist_ok=True)

        filename = "nidibot.log"
        filepath = os.path.join(log_directory, filename)

        handler = TimedRotatingFileHandler(filepath, when="midnight", backupCount=60)
        handler.suffix = "%Y%m%d"

        logging.basicConfig(
            level=logging.DEBUG,
            format="%(asctime)s:%(levelname)s:%(funcName)s %(message)s",
            handlers=[handler, logging.StreamHandler()],
        )

        self.__configuration = self.__parse_configuration_json_file()

        self.__gameserver = Nitrado(self.__configuration.connection.nitrado_api_token)

        self.__bot = lightbulb.BotApp(
            token=self.__configuration.connection.discord_bot_token
        )

        @self.__bot.listen(hikari.StartedEvent)
        async def on_started(_) -> None:
            logging.debug("Starting the bot.")
            # logging.debug("Bot is logged in as '%s' user.", self.__bot.user.name)

            logging.debug("Waiting for command tree sync.")
            # await self.__bot.tree.sync()
            logging.debug("Command tree sync is completed.")

        @self.__bot.command
        @lightbulb.command(
            name="status",
            description="Provides extended information about game server status.",
        )
        @lightbulb.implements(lightbulb.SlashCommand)
        async def status(ctx) -> None:
            logging.debug("Called 'status' by '%s'.", ctx.author)

            server_status = self.__gameserver.get_status()

            title = f"{server_status.game_name}"
            if server_status.game_version:
                title += f" ({server_status.game_version})"

            if server_status.status == "online":
                status_smiley = ":white_check_mark:"
                status_color = hikari.colors.Color(0x37CB78)
            elif server_status.status == "offline":
                status_smiley = ":no_entry:"
                status_color = hikari.colors.Color(0xE64A42)
            elif server_status.status == "restarting":
                status_smiley = ":warning:"
                status_color = hikari.colors.Color(0xE67E22)

            embed = hikari.Embed(
                title=title,
                color=status_color,
            )

            embed.add_field(
                name="Address:", value=f"`{server_status.server_address}`", inline=True
            )
            embed.add_field(
                name="Status:",
                value=f"{status_smiley} {server_status.status}",
                inline=True,
            )

            players = (
                f"{server_status.players_connected} / {server_status.players_limit}"
            )
            if len(server_status.player_names) > 0:
                players += f" ({server_status.player_names})"
            embed.add_field(name="Players:", value=f"{players}", inline=True)

            update_smiley = ""
            if server_status.update_available:
                update_smiley = ":warning:"
            else:
                update_smiley = ":white_check_mark:"

            date_time = server_status.available_until.split(" ")
            date_parts = date_time[0].split("-")

            d0 = date(datetime.now().year, datetime.now().month, datetime.now().day)
            d1 = date(int(date_parts[0]), int(date_parts[1]), int(date_parts[2]))
            delta = d1 - d0

            embed.add_field(
                name="Available until:",
                value=f"{server_status.available_until} ({delta.days} days left)",
                inline=True,
            )
            embed.add_field(
                name="Update available:",
                value=f"{update_smiley} {'yes' if server_status.update_available else 'no'}",
                inline=True,
            )

            await ctx.respond(embed=embed)

        @self.__bot.command
        @lightbulb.command(
            name="start",
            description="Starts server if it is offline, restarts server if it is online.",
        )
        @lightbulb.implements(lightbulb.SlashCommand)
        async def start(ctx) -> None:
            logging.debug("Called 'start' by '%s'.", ctx.author)

            user = str(ctx.author)
            if user not in self.__configuration.discord_admin_users:
                embed = hikari.Embed(
                    description="Sorry but you don't have rights to call this command! :liar:",
                    color=hikari.colors.Color(0xE64A42),
                )

                await ctx.respond(embed=embed)
                return

            embed = hikari.Embed(
                description=":warning: Starting server! :warning:",
                color=hikari.colors.Color(0xE64A42),
            )
            await ctx.respond(embed=embed)

            self.__gameserver.start()

        @self.__bot.command
        @lightbulb.command(
            name="stop",
            description="Stops server if it is online.",
        )
        @lightbulb.implements(lightbulb.SlashCommand)
        async def stop(ctx) -> None:
            logging.debug("Called 'stop' by '%s'.", ctx.author)

            user = str(ctx.author)
            if user not in self.__configuration.discord_admin_users:
                embed = hikari.Embed(
                    description="Sorry but you don't have rights to call this command! :liar:",
                    color=hikari.colors.Color(0xE64A42),
                )

                await ctx.respond(embed=embed)
                return

            embed = hikari.Embed(
                description=":warning: Stopping server! :warning:",
                color=hikari.colors.Color(0xE64A42),
            )
            await ctx.respond(embed=embed)

            self.__gameserver.stop()

        @self.__bot.command
        @lightbulb.command(
            name="restart",
            description="Restarts server if it is online, starts server if it is offline.",
        )
        @lightbulb.implements(lightbulb.SlashCommand)
        async def restart(ctx) -> None:
            logging.debug("Called 'restart' by '%s'.", ctx.author)

            user = str(ctx.author)
            if user not in self.__configuration.discord_admin_users:
                embed = hikari.Embed(
                    description="Sorry but you don't have rights to call this command! :liar:",
                    color=hikari.colors.Color(0xE64A42),
                )

                await ctx.respond(embed=embed)
                return

            embed = hikari.Embed(
                description=":warning: Restarting server! :warning:",
                color=hikari.colors.Color(0xE64A42),
            )
            await ctx.respond(embed=embed)

            self.__gameserver.restart()

        @self.__bot.command
        @lightbulb.command(
            name="backup",
            description="Creates backup of games server files and uploads them to storage.",
        )
        @lightbulb.implements(lightbulb.SlashCommand)
        async def backup(ctx) -> None:
            logging.debug("Called 'backup' by '%s'.", ctx.author)

            user = str(ctx.author)
            if user not in self.__configuration.discord_admin_users:
                embed = hikari.Embed(
                    description="Sorry but you don't have rights to call this command! :liar:",
                    color=hikari.colors.Color(0xE64A42),
                )

                await ctx.respond(embed=embed)
                return

            embed = hikari.Embed(
                description=":no_entry: This command is not yet implemented! :no_entry:",
                color=hikari.colors.Color(0xE64A42),
            )

            await ctx.respond(embed=embed)

    def __parse_configuration_json_file(self) -> NidibotConfiguration:
        configuration_filepath = os.path.join(
            self.__working_folder_path, "bot_configuration.json"
        )

        with open(configuration_filepath, "r", encoding="utf-8") as json_file:
            configuration_json = json.load(json_file)

        configuration = from_dict(
            data_class=NidibotConfiguration,
            data=configuration_json,
        )

        return configuration

    @staticmethod
    def initialize_folder() -> None:
        """ """
        current_script_folder = os.path.dirname(os.path.realpath(__file__))
        current_working_folder = os.getcwd()
        shutil.copytree(
            os.path.join(current_script_folder, "templates", "common"),
            current_working_folder,
            dirs_exist_ok=True,
        )

        if platform.system() == "Linux":
            shutil.copytree(
                os.path.join(current_script_folder, "templates", "linux"),
                current_working_folder,
                dirs_exist_ok=True,
            )

    def activate(self) -> None:
        self.__bot.run()



import json
import os

from nidibot.nidibot import Nidibot


def test_folder_initialization(tmp_path):
    os.chdir(tmp_path)

    assert len(os.listdir(tmp_path)) == 0

    Nidibot.initialize_folder()

    assert len(os.listdir(tmp_path)) > 0


def test_discord_connection(tmp_path):
    os.chdir(tmp_path)

    json_configuration = {
        "discord_bot_token": "",
        "nitrado_api_token": "",
        "nitrado_service_id": "",
    }

    configuration_filepath = os.path.join(tmp_path, "bot_configuration.json")
    with open(configuration_filepath, "w", encoding="utf-8") as file:
        json.dump(
            json_configuration,
            file,
            ensure_ascii=False,
            indent=4,
        )

    bot = Nidibot(tmp_path)
    bot.activate()



#!/usr/bin/env python

import logging
import os
import signal
import sys

from nidibot.nidibot import Nidibot


def shutdown_signal_handler(_1, _2):
    logging.critical("Caught SIGINT signal, nidibot will be shut down.")
    sys.exit(0)


if __name__ == "__main__":
    #
    # Catch Ctrl+C signal for notifying about bureau shutdown.
    #
    signal.signal(signal.SIGINT, shutdown_signal_handler)

    bot = Nidibot(os.path.dirname(os.path.realpath(__file__)))
    bot.activate()



from nidibot.nitrado import Nitrado

nitrado_api_token = ""


def test_nitrado_api_availability():
    nitrado = Nitrado(nitrado_api_token)
    assert nitrado.is_api_available()


def test_nitrado_maintenance():
    nitrado = Nitrado(nitrado_api_token)
    assert not nitrado.is_maintenance_ongoing()


def test_gameserver_statuses():
    nitrado = Nitrado(nitrado_api_token)
    assert len(nitrado.get_statuses()) > 0


def test_gameserver_start():
    nitrado = Nitrado(nitrado_api_token)
    assert nitrado.start()


def test_gameserver_stop():
    nitrado = Nitrado(nitrado_api_token)
    assert nitrado.stop()


def test_gameserver_restart():
    nitrado = Nitrado(nitrado_api_token)
    assert nitrado.restart()



#!/usr/bin/env python

from dataclasses import dataclass, field
import json
import logging
import requests


@dataclass
class NitradoService:
    id: str = ""
    short_name: str = ""
    available_until: str = ""


@dataclass
class NitradoServerStatus:
    game_name: str = ""
    game_version: str = ""
    server_address: str = ""
    status: str = ""
    available_until: str = ""
    update_available: bool = False
    players_limit: int = 0
    players_connected: int = 0
    player_names: list = field(default_factory=list)


@dataclass
class NitradoFtpConfiguration:
    hostname: str = ""
    port: int = 21
    username: str = ""
    password: str = ""


class Nitrado:
    def __init__(self, api_token: str):
        self.__api_token = api_token
        self.__default_timeout_seconds = 10

        self.__service_list = self.__get_service_list()
        self.__ftp_configuration = self.__get_ftp_configuration()

        logging.basicConfig(
            level=logging.DEBUG,
            format="%(asctime)s:%(levelname)s:%(funcName)s %(message)s",
            handlers=[logging.StreamHandler()],
        )

    def __get_service_list(self) -> list:
        service_list = []

        try:
            headers = {"Authorization": self.__api_token}
            response = requests.get(
                "https://api.nitrado.net/services",
                timeout=self.__default_timeout_seconds,
                headers=headers,
            )

            logging.debug(
                "Response received, status code: %d, content: '%s'.",
                response.status_code,
                response.content.decode("utf-8"),
            )

            content_json = json.loads(response.content.decode("utf-8"))
            data_service_list = content_json["data"]["services"]
            for data_service in data_service_list:
                service: NitradoService = NitradoService()

                service.id = data_service["id"]
                service.short_name = data_service["details"]["folder_short"]
                service.available_until = str(data_service["suspend_date"]).replace(
                    "T", " "
                )

                service_list.append(service)

        except Exception as exception:
            logging.exception(exception)

        return service_list

    def __get_ftp_configuration(self) -> NitradoFtpConfiguration:
        ftp_configuration: NitradoFtpConfiguration = NitradoFtpConfiguration()

        try:
            headers = {"Authorization": self.__api_token}
            response = requests.get(
                f"https://api.nitrado.net/services/{self.__service_list[0].id}/gameservers",
                timeout=self.__default_timeout_seconds,
                headers=headers,
            )

            logging.debug(
                "Response received, status code: %d.",
                response.status_code,
            )

            content_json = json.loads(response.content.decode("utf-8"))
            ftp_dict = content_json["data"]["gameserver"]["credentials"]["ftp"]

            ftp_configuration.hostname = ftp_dict["hostname"]
            ftp_configuration.port = int(ftp_dict["port"])
            ftp_configuration.username = ftp_dict["username"]
            ftp_configuration.password = ftp_dict["password"]

        except Exception as exception:
            logging.exception(exception)

        return ftp_configuration

    def is_api_available(self) -> bool:
        try:
            response = requests.get(
                "https://api.nitrado.net/ping", timeout=self.__default_timeout_seconds
            )

            logging.debug(
                "Response received, status code: %d, content: '%s'.",
                response.status_code,
                response.content.decode("utf-8"),
            )

            return response.status_code == 200

        except Exception as exception:
            logging.exception(exception)
            return False

    def is_maintenance_ongoing(self) -> bool:
        try:
            response = requests.get(
                "https://api.nitrado.net/maintenance",
                timeout=self.__default_timeout_seconds,
            )

            logging.debug(
                "Response received, status code: %d, content: '%s'.",
                response.status_code,
                response.content.decode("utf-8"),
            )

            content_json = json.loads(response.content.decode("utf-8"))
            maintenance_dict = content_json["data"]["maintenance"]

            return response.status_code == 200 and (
                bool(maintenance_dict["cloud_backend"])
                or bool(maintenance_dict["domain_backend"])
                or bool(maintenance_dict["dns_backend"])
                or bool(maintenance_dict["pmacct_backend"])
            )

        except Exception as exception:
            logging.exception(exception)
            return True

    def get_status(self, server_index: int = 0) -> NitradoServerStatus:
        server_status: NitradoServerStatus = NitradoServerStatus()

        try:
            headers = {"Authorization": self.__api_token}
            response = requests.get(
                f"https://api.nitrado.net/services/{self.__service_list[server_index].id}/gameservers",
                timeout=self.__default_timeout_seconds,
                headers=headers,
            )

            logging.debug(
                "Response received, status code: %d.",
                response.status_code,
            )

            content_json = json.loads(response.content.decode("utf-8"))
            gameserver_dict = content_json["data"]["gameserver"]
            query_dict = gameserver_dict["query"]

            server_status: NitradoServerStatus = NitradoServerStatus()
            server_status.game_name = gameserver_dict["game_human"]
            server_status.update_available = (
                gameserver_dict["game_specific"]["update_status"] != "up_to_date"
            )
            server_status.available_until = self.__service_list[
                server_index
            ].available_until

            if gameserver_dict["status"] == "started":
                server_status.status = "online"
            elif gameserver_dict["status"] == "stopped":
                server_status.status = "offline"
            elif gameserver_dict["status"] == "starting":
                server_status.status = "starting"

            if len(query_dict) > 0:
                server_status.game_version = gameserver_dict["query"]["version"]
                server_status.server_address = gameserver_dict["query"]["connect_ip"]
                server_status.players_limit = int(
                    gameserver_dict["query"]["player_max"]
                )
                server_status.players_connected = int(
                    gameserver_dict["query"]["player_current"]
                )
                server_status.player_names = gameserver_dict["query"]["players"]
            else:
                server_status.server_address = (
                    str(gameserver_dict["ip"])
                    + ":"
                    + str(gameserver_dict["query_port"])
                )
                server_status.players_limit = int(gameserver_dict["slots"])
                server_status.player_names = []

            return server_status

        except Exception as exception:
            logging.exception(exception)

        return NitradoServerStatus()

    def get_statuses(self) -> list:
        statuses = []

        counter = 0
        for _ in self.__service_list:
            statuses.append(self.get_status(counter))
            counter += 1

        return statuses

    def start(self, server_index: int = 0) -> bool:
        return self.restart(server_index)

    def stop(self, server_index: int = 0) -> bool:
        try:
            headers = {"Authorization": self.__api_token}
            response = requests.post(
                f"https://api.nitrado.net/services/{self.__service_list[server_index].id}/gameservers/stop",
                timeout=self.__default_timeout_seconds,
                headers=headers,
            )

            logging.debug(
                "Response received, status code: %d, content: %s.",
                response.status_code,
                response.content.decode("utf-8"),
            )

            content_json = json.loads(response.content.decode("utf-8"))

            return response.status_code == 200 and content_json["status"] == "success"

        except Exception as exception:
            logging.exception(exception)
            return False

    def restart(self, server_index: int = 0) -> bool:
        try:
            headers = {"Authorization": self.__api_token}
            response = requests.post(
                f"https://api.nitrado.net/services/{self.__service_list[server_index].id}/gameservers/restart",
                timeout=self.__default_timeout_seconds,
                headers=headers,
            )

            logging.debug(
                "Response received, status code: %d, content: %s.",
                response.status_code,
                response.content.decode("utf-8"),
            )

            content_json = json.loads(response.content.decode("utf-8"))

            return response.status_code == 200 and content_json["status"] == "success"

        except Exception as exception:
            logging.exception(exception)
            return False
